import { Service } from "typedi";
import db from "..";
import type { ResourceModel } from "../schema/resource.schema";
import type { EducationalStagesModel } from "../schema/educational-stage.schema";
import resourceEducationalStagesTable from "../relations/resource-educational-stage.relation";
import { and, eq, inArray } from "drizzle-orm";
import educationalStageTable from "../schema/educational-stage.schema";
import resourceTable from "../schema/resource.schema";



@Service()
export class resourceEducationalStages {

    async associateResourceWithEducationalStages(resourceId: ResourceModel['id'], educationalStageIds: EducationalStagesModel['id'][]): Promise<void> {
        try {
            const existingAssociations = await db
                .select({
                    educational_stage_id: resourceEducationalStagesTable.educational_stage_id
                })
                .from(resourceEducationalStagesTable)
                .where(eq(resourceEducationalStagesTable.resource_id, resourceId))
                .execute();

            const existingEducationalStageIds = existingAssociations.map(row => row.educational_stage_id);
            const newEducationalStageIds = educationalStageIds.filter(educationalStageId => !existingEducationalStageIds.includes(educationalStageId));

            if (newEducationalStageIds.length > 0) {
                const valuesToInsert = newEducationalStageIds.map(educationalStageId => ({
                    resource_id: resourceId,
                    educational_stage_id: educationalStageId
                }));

                await db
                    .insert(resourceEducationalStagesTable)
                    .values(valuesToInsert)
                    .onConflictDoNothing()
                    .execute();

                console.log(`Resource ${resourceId} associated with new educational stages: ${newEducationalStageIds.join(", ")}`);
            } else {
                console.log(`No new educational stages to associate for resource ${resourceId}`);
            }
        } catch (error) {
            console.error("Error associating resource with educational stages:", error);
        }
    }



    async getEducationalStagesByResource(resourceId: ResourceModel['id']): Promise<EducationalStagesModel[]> {
        try {
            const educationalStages = await db
                .select()
                .from(resourceEducationalStagesTable)
                .innerJoin(educationalStageTable, eq(resourceEducationalStagesTable.educational_stage_id, educationalStageTable.id))
                .where(eq(resourceEducationalStagesTable.resource_id, resourceId))
                .execute();

            return educationalStages.map(row => row.educational_stages);
        } catch (error) {
            console.error("Error getting educational stages by resource:", error);
            return [];
        }
    }

    async removeEducationalStageFromResource(resourceId: ResourceModel['id'], educationalStageId: EducationalStagesModel['id']): Promise<void> {
        try {
            await db
                .delete(resourceEducationalStagesTable)
                .where(
                    and(
                        eq(resourceEducationalStagesTable.resource_id, resourceId),
                        eq(resourceEducationalStagesTable.educational_stage_id, educationalStageId)
                    )
                )
                .execute();

            console.log(`Educational stage ${educationalStageId} removed from resource ${resourceId}`);
        } catch (error) {
            console.error("Error removing educational stage from resource:", error);
        }
    }


    async isAssociationExists(resourceId: ResourceModel['id'], educationalStageId: EducationalStagesModel['id']): Promise<boolean> {
        try {
            const result = await db
                .select()
                .from(resourceEducationalStagesTable)
                .where(
                    and(
                        eq(resourceEducationalStagesTable.resource_id, resourceId),
                        eq(resourceEducationalStagesTable.educational_stage_id, educationalStageId)
                    )
                )
                .execute();

            return result.length > 0;
        } catch (error) {
            console.error("Error checking if educational stage is associated with resource:", error);
            return false;
        }
    }

    async getResourcesByEducationalStage(educationalStageId: EducationalStagesModel['id']): Promise<ResourceModel[]> {
        try {
            const resources = await db
                .select()
                .from(resourceEducationalStagesTable)
                .innerJoin(resourceTable, eq(resourceEducationalStagesTable.resource_id, resourceTable.id))
                .where(eq(resourceEducationalStagesTable.educational_stage_id, educationalStageId))
                .execute();

            return resources.map(row => row.resource);
        } catch (error) {
            console.error("Error getting resources by educational stage:", error);
            return [];
        }
    }


    async updateResourceEducationalStages(resourceId: ResourceModel['id'], newEducationalStageIds: EducationalStagesModel['id'][]): Promise<void> {
        const existingAssociations = await db
            .select({
                educational_stage_id: resourceEducationalStagesTable.educational_stage_id
            })
            .from(resourceEducationalStagesTable)
            .where(eq(resourceEducationalStagesTable.resource_id, resourceId))
            .execute();

        const existingEducationalStageIds = existingAssociations.map(row => row.educational_stage_id);
        const educationalStageToAdd = newEducationalStageIds.filter(educationalStageId => !existingEducationalStageIds.includes(educationalStageId));
        const educationalStageToRemove = existingEducationalStageIds.filter(educationalStageId => !newEducationalStageIds.includes(educationalStageId));

        if (educationalStageToAdd.length > 0) {
            const valuesToInsert = educationalStageToAdd.map(educationalStageId => ({
                resource_id: resourceId,
                educational_stage_id: educationalStageId
            }));

            await db.insert(resourceEducationalStagesTable).values(valuesToInsert).execute();
        }

        if (educationalStageToRemove.length > 0) {
            await db
                .delete(resourceEducationalStagesTable)
                .where(
                    and(
                        eq(resourceEducationalStagesTable.resource_id, resourceId),
                        inArray(resourceEducationalStagesTable.educational_stage_id, educationalStageToRemove)
                    )
                )
                .execute();
        }
    }
}