import { Service } from "typedi";
import db from "..";
import type { ResourceModel } from "../schema/resource.schema";
import resourceLanguagesTable from "../relations/resource-language.relation";
import type { LanguageModel } from "../schema/language.schema";
import languageTable from "../schema/language.schema";
import { and, eq, inArray } from "drizzle-orm";
import resourceTable from "../schema/resource.schema";



@Service()
export class resourceLanguages {
    //associar um recurso a varias linguagens
    async associateResourceWithLanguages(resourceId: ResourceModel['id'], languageIds: LanguageModel['id'][]): Promise<void> {
        try {
            // Passo 1: Buscar linguagens já associadas ao recurso
            const existingAssociations = await db
                .select({
                    language_id: resourceLanguagesTable.language_id
                })
                .from(resourceLanguagesTable)
                .where(eq(resourceLanguagesTable.resource_id, resourceId))
                .execute();
    
            // Extrair os IDs das linguagens já associadas
            const existingLanguageIds = existingAssociations.map(row => row.language_id);
    
            // Passo 2: Filtrar linguagens que ainda não estão associadas
            const newLanguageIds = languageIds.filter(languageId => !existingLanguageIds.includes(languageId));
    
            // Passo 3: Associar novas linguagens (se houver alguma nova, nao deleta as que ja existem)
            if (newLanguageIds.length > 0) {
                const valuesToInsert = newLanguageIds.map(languageId => ({
                    resource_id: resourceId,
                    language_id: languageId
                }));
    
                await db
                    .insert(resourceLanguagesTable)
                    .values(valuesToInsert)
                    .onConflictDoNothing() // Evita tentativas de duplicação
                    .execute();
                
                console.log(`Resource ${resourceId} associated with new languages: ${newLanguageIds.join(", ")}`);
            } else {
                console.log(`No new languages to associate for resource ${resourceId}`);
            }
        } catch (error) {
            console.error("Error associating resource with languages:", error);
        }
    }

    // Obter todas as linguagens de um recurso
    async getLanguagesByResource(resourceId: ResourceModel['id']): Promise<LanguageModel[]> {
        try {
            const languages = await db
                .select()
                .from(languageTable)
                .innerJoin(resourceLanguagesTable, eq(resourceLanguagesTable.language_id, languageTable.id))
                .where(eq(resourceLanguagesTable.resource_id, resourceId))
                .execute();

            return languages.map(row => row.language);
        } catch (error) {
            console.error("Error fetching languages for resource:", error);
            throw error;
        }
    }

    // // Remover associação entre um recurso e uma linguagem
    // async removeLanguageFromResource(resourceId: ResourceModel['id'], languageId: LanguageModel['id']): Promise<void> {
    //     try {
    //         await db
    //             .delete(resourceLanguagesTable)
    //             .where(
    //                 and(
    //                     eq(resourceLanguagesTable.resource_id, resourceId),
    //                     eq(resourceLanguagesTable.language_id, languageId)
    //                 ))
    //             .execute();

    //         console.log(`Language ${languageId} removed from resource ${resourceId}`);
    //     } catch (error) {
    //         console.error("Error removing language from resource:", error);
    //         throw error;
    //     }
    // }

    // Remover associações entre um recurso e várias linguagens
    async removeLanguageFromResource(resourceId: ResourceModel['id'], languageIds: LanguageModel['id'][]): Promise<void> {
        try {
            await db
                .delete(resourceLanguagesTable)
                .where(
                    and(
                        eq(resourceLanguagesTable.resource_id, resourceId),
                        inArray(resourceLanguagesTable.language_id, languageIds)
                    )
                )
                .execute();

            console.log(`Languages ${languageIds.join(", ")} removed from resource ${resourceId}`);
        } catch (error) {
            console.error("Error removing languages from resource:", error);
            throw error;
        }
    }


    //retorna true se a associação existe
    async isAssociationExists(resourceId: ResourceModel['id'], languageId: LanguageModel['id']): Promise<boolean> {
        try {
            const result = await db
                .select()
                .from(resourceLanguagesTable)
                .where(
                    and(
                        eq(resourceLanguagesTable.resource_id, resourceId),
                        eq(resourceLanguagesTable.language_id, languageId)
                    ))
                .execute();

            return result.length > 0;
        } catch (error) {
            console.error("Error checking association between resource and language:", error);
            throw error;
        }
    }


    // Obter todos os recursos associados a uma linguagem
    async getResourcesByLanguage(languageId: LanguageModel['id']): Promise<ResourceModel[]> {
        try {
            const resources = await db
                .select()
                .from(resourceLanguagesTable)
                .innerJoin(resourceTable, eq(resourceLanguagesTable.resource_id, resourceTable.id))
                .where(eq(resourceLanguagesTable.language_id, languageId))
                .execute();

            return resources.map(row => row.resource);
        } catch (error) {
            console.error("Error fetching resources for language:", error);
            throw error;
        }
    }

    // Atualiza as associações de linguagens de um recurso
    async updateResourceLanguages(resourceId: number, newLanguageIds: number[]): Promise<void> {
        // 1: Busca todas as associações atuais do recurso
        const existingAssociations = await db
            .select({ language_id: resourceLanguagesTable.language_id })
            .from(resourceLanguagesTable)
            .where(eq(resourceLanguagesTable.resource_id, resourceId))
            .execute();

        const existingLanguageIds = existingAssociations.map(assoc => assoc.language_id);
        // 2: Identificar quais associações precisam ser adicionadas e quais podem ser mantidas
        const languagesToAdd = newLanguageIds.filter(id => !existingLanguageIds.includes(id));
        const languagesToRemove = existingLanguageIds.filter(id => !newLanguageIds.includes(id));

        // 3: Adiciona novas associações
        if (languagesToAdd.length > 0) {
            const associationsToAdd = languagesToAdd.map(languageId => ({
                resource_id: resourceId,
                language_id: languageId,
            }));
            await db.insert(resourceLanguagesTable).values(associationsToAdd).execute();
        }

        // 4: Remover associações que não foram passadas
        if (languagesToRemove.length > 0) {
            await db
                .delete(resourceLanguagesTable)
                .where(
                    and(
                        eq(resourceLanguagesTable.resource_id, resourceId),
                        inArray(resourceLanguagesTable.language_id, languagesToRemove)
                    )
                )
                .execute();
        }
    }
}