import { boolean, integer, pgTable, serial, text, timestamp } from "drizzle-orm/pg-core";
import userTable from "./user.schema";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";
import resourceTable from "./resource.schema";
import collectionTable from "./collection.schema";
import stateEnum from "./stateEnum.schema";

const complaintTable = pgTable('complaints', {
  id: serial('id').primaryKey()
    .notNull()
    .unique(),
  state: stateEnum('state')
    .notNull()
    .default('under_review'),
  description: text('description')
    .notNull(),
  denouncer_id: integer('denouncer_id')
    .notNull()
    .references(() => userTable.id, {onDelete: 'cascade'}),
  resource_id: integer('resource_id')
    .notNull()
    .references(() => resourceTable.id, {onDelete: 'cascade'}),
  collection_id: integer('collection_id')
    .references(() => collectionTable.id, {onDelete: 'cascade'}),
  user_id: integer('user_id')
    .references(() => userTable.id, {onDelete: 'cascade'}),
  evaluated_at: timestamp('evaluated_at'),
  created_at: timestamp('created_at')
    .notNull()
    .defaultNow(),
  q1: boolean('q1'),
  q2: boolean('q2'),
  q3: boolean('q3'),
  q4: boolean('q4'),
})

const complaintModelSchema = createSelectSchema(complaintTable)
const complaintDtoSchema = complaintModelSchema
const complaintInputSchema = createInsertSchema(complaintTable)
const complaintUpdateSchema = complaintInputSchema.partial().required({ id: true, user_id: true})

export type ComplaintModel = z.infer<typeof complaintModelSchema>
export type ComplaintDto = z.infer<typeof complaintDtoSchema>
export type ComplaintInput = z.infer<typeof complaintInputSchema>
export type ComplaintUpdate = z.infer<typeof complaintUpdateSchema>

export const complaintSchemas = {
  complaintModelSchema,
  complaintDtoSchema,
  complaintInputSchema,
  complaintUpdateSchema,
}

export default complaintTable