import { sql } from 'drizzle-orm'
import { boolean, integer, pgTable, serial, text, timestamp, varchar } from 'drizzle-orm/pg-core'
import { createInsertSchema, createSelectSchema } from 'drizzle-zod'
import { z } from 'zod'
import userStatsTable from './user-stats.schema'

const userTable = pgTable('users', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  name: varchar('name', { length: 255 })
    .notNull(),
  username: varchar('username', { length: 255 })
    .unique()
    .notNull(),
  password: varchar('password', { length: 255 })
    .notNull(),
  email: varchar('email', { length: 255 })
    .unique()
    .notNull(),
  description: text('description')
    .default("sem descrição"),
  institution: text('institution')
    .default("sem instituição"),
  birthday: timestamp('birthday', { mode: 'string' })
    .notNull(),
  cpf: varchar('cpf', { length: 11 })
    .unique()
    .notNull(),
  createdAt: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updatedAt: timestamp('updated_at', { mode: 'string' })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),  
  confirmed_at: timestamp('confirmed_at', { mode: 'string' }),
  confirmation_sent_at: timestamp('confirmation_sent_at', { mode: 'string' }),
  deleted_at: timestamp('deleted_at', { mode: 'string' }),
  reactivated_at: timestamp('reactivated_at', { mode: 'string' }),
  active: boolean('active')
    .default(true),
  user_stats_id: integer('user_stats_id')
    .references(() => userStatsTable.id)
    .unique()
    .notNull(),
})

const userInputSchema = createInsertSchema(userTable)
.partial()
.required({
  name: true,
  username: true,
  password: true,
  email: true,
  birthday: true,
  cpf: true
})
const userModelSchema = createSelectSchema(userTable)
const userDtoSchema = createSelectSchema(userTable).omit({
  password: true
})
const userUpdateSchema = createSelectSchema(userTable)
  .partial()
  .required({ id: true })
const userProfileSchema = userModelSchema.omit({
  id: true,
  user_stats_id: true,
  password: true,
  birthday: true,
  cpf: true,
  createdAt: true,
  updatedAt: true,
  confirmed_at: true,
  confirmation_sent_at: true,
  deleted_at: true,
  reactivated_at: true,
  active: true,
  
})

export type UserInput = z.infer<typeof userInputSchema>
export type UserUpdate = z.infer<typeof userUpdateSchema>
export type UserModel = z.infer<typeof userModelSchema>
export type UserDto = z.infer<typeof userDtoSchema>
export type UserProfile = z.infer<typeof userProfileSchema>

export const userSchemas = {
  input: userInputSchema,
  update: userUpdateSchema,
  model: userModelSchema,
  dto: userDtoSchema,
  profile: userProfileSchema
}

export default userTable
